@@ -28,7 +28,7 @@
 #include "object.h"
 #include "boxed.h"
 #include "union.h"
-#include <gjs/gjs.h>
+#include <gjs/gjs-module.h>
 #include <gjs/compat.h>
 
 #include <util/log.h>
@@ -77,7 +77,7 @@ static struct {
 } global_destroy_trampoline;
 
 typedef struct {
-    JSContext *context;
+    JSRuntime *runtime;
     GICallableInfo *info;
     jsval js_function;
     ffi_cif cif;
@@ -128,7 +128,11 @@ function_new_resolve(JSContext *context,
 static void
 gjs_callback_trampoline_free(GjsCallbackTrampoline *trampoline)
 {
-    JS_RemoveValueRoot(trampoline->context, &trampoline->js_function);
+    JSContext *context;
+
+    context = gjs_runtime_get_current_context(trampoline->runtime);
+
+    JS_RemoveValueRoot(context, &trampoline->js_function);
     g_callable_info_free_closure(trampoline->info, trampoline->closure);
     g_base_info_unref( (GIBaseInfo*) trampoline->info);
     g_slice_free(GjsCallbackTrampoline, trampoline);
@@ -147,6 +151,7 @@ gjs_callback_closure(ffi_cif *cif,
                      void **args,
                      void *data)
 {
+    JSContext *context;
     GjsCallbackTrampoline *trampoline;
     int i, n_args, n_jsargs;
     jsval *jsargs, rval;
@@ -156,6 +161,9 @@ gjs_callback_closure(ffi_cif *cif,
     trampoline = data;
     g_assert(trampoline);
 
+    context = gjs_runtime_get_current_context(trampoline->runtime);
+    JS_BeginRequest(context);
+
     n_args = g_callable_info_get_n_args(trampoline->info);
 
     g_assert(n_args >= 0);
@@ -172,14 +180,14 @@ gjs_callback_closure(ffi_cif *cif,
         if (g_type_info_get_tag(&type_info) == GI_TYPE_TAG_VOID)
             continue;
 
-        if (!gjs_value_from_g_argument(trampoline->context,
+        if (!gjs_value_from_g_argument(context,
                                        &jsargs[n_jsargs++],
                                        &type_info,
                                        args[i]))
             goto out;
     }
 
-    if (!JS_CallFunctionValue(trampoline->context,
+    if (!JS_CallFunctionValue(context,
                               NULL,
                               trampoline->js_function,
                               n_jsargs,
@@ -190,7 +198,7 @@ gjs_callback_closure(ffi_cif *cif,
 
     g_callable_info_load_return_type(trampoline->info, &ret_type);
 
-    if (!gjs_value_to_g_argument(trampoline->context,
+    if (!gjs_value_to_g_argument(context,
                                  rval,
                                  &ret_type,
                                  "callback",
@@ -205,16 +213,18 @@ gjs_callback_closure(ffi_cif *cif,
 
 out:
     if (!success) {
-        gjs_log_exception (trampoline->context, NULL);
+        gjs_log_exception (context, NULL);
 
         /* Fill in the result with some hopefully neutral value */
         g_callable_info_load_return_type(trampoline->info, &ret_type);
-        gjs_g_argument_init_default (trampoline->context, &ret_type, result);
+        gjs_g_argument_init_default (context, &ret_type, result);
     }
 
     if (trampoline->scope == GI_SCOPE_TYPE_ASYNC) {
         completed_trampolines = g_slist_prepend(completed_trampolines, trampoline);
     }
+
+    JS_EndRequest(context);
 }
 
 /* The global entry point for any invocations of GDestroyNotify;
@@ -268,7 +278,7 @@ gjs_callback_trampoline_new(JSContext      *context,
     g_assert(JS_TypeOfValue(context, function) == JSTYPE_FUNCTION);
 
     trampoline = g_slice_new(GjsCallbackTrampoline);
-    trampoline->context = context;
+    trampoline->runtime = JS_GetRuntime(context);
     trampoline->info = callable_info;
     g_base_info_ref((GIBaseInfo*)trampoline->info);
     trampoline->js_function = function;
@@ -562,11 +572,15 @@ gjs_invoke_c_function(JSContext      *context,
          * separately */
     }
 
+    gjs_runtime_push_context(JS_GetRuntime(context), context);
+
     g_assert_cmpuint(in_args_pos, ==, (guint8)function->invoker.cif.nargs);
     g_assert_cmpuint(inout_args_pos, ==, inout_args_len);
     g_assert_cmpuint(out_args_pos, ==, out_args_len);
     ffi_call(&(function->invoker.cif), function->invoker.native_address, &return_value, in_arg_pointers);
 
+    gjs_runtime_pop_context(JS_GetRuntime(context));
+
     /* Return value and out arguments are valid only if invocation doesn't
      * return error. In arguments need to be released always.
      */
@@ -979,7 +993,7 @@ function_new(JSContext      *context,
     Function *priv;
 
     /* put constructor for GIRepositoryFunction() in the global namespace */
-    global = JS_GetGlobalObject(context);
+    global = gjs_get_import_global(context);
 
     if (!gjs_object_has_property(context, global, gjs_function_class.name)) {
         JSObject *prototype;
@@ -1018,7 +1032,7 @@ function_new(JSContext      *context,
                   gjs_function_class.name, prototype);
     }
 
-    function = JS_ConstructObject(context, &gjs_function_class, NULL, NULL);
+    function = JS_ConstructObject(context, &gjs_function_class, NULL, global);
     if (function == NULL) {
         gjs_debug(GJS_DEBUG_GFUNCTION, "Failed to construct function");
         return NULL;
@@ -1037,16 +1051,14 @@ gjs_define_function(JSContext      *context,
                     GIFunctionInfo *info)
 {
     JSObject *function;
-    JSContext *load_context;
 
-    load_context = gjs_runtime_get_load_context(JS_GetRuntime(context));
-    JS_BeginRequest(load_context);
+    JS_BeginRequest(context);
 
-    function = function_new(load_context, info);
+    function = function_new(context, info);
     if (function == NULL) {
-        gjs_move_exception(load_context, context);
+        gjs_move_exception(context, context);
 
-        JS_EndRequest(load_context);
+        JS_EndRequest(context);
         return NULL;
     }
 
@@ -1057,11 +1069,11 @@ gjs_define_function(JSContext      *context,
                            GJS_MODULE_PROP_FLAGS)) {
         gjs_debug(GJS_DEBUG_GFUNCTION, "Failed to define function");
 
-        JS_EndRequest(load_context);
+        JS_EndRequest(context);
         return NULL;
     }
 
-    JS_EndRequest(load_context);
+    JS_EndRequest(context);
     return function;
 }
 
