@@ -27,7 +27,7 @@
 
 #include "param.h"
 #include "repo.h"
-#include <gjs/gjs.h>
+#include <gjs/gjs-module.h>
 #include <gjs/compat.h>
 
 #include <util/log.h>
