@@ -25,7 +25,8 @@
 #include <string.h>
 #include <glib.h>
 #include "byteArray.h"
-#include <gjs/gjs.h>
+#include <gjs/gjs-module.h>
+#include <gjs/compat.h>
 #include <util/log.h>
 #include <jsapi.h>
 
@@ -39,15 +40,15 @@ GJS_DEFINE_PRIV_FROM_JS(ByteArrayInstance, gjs_byte_array_class)
 
 static JSBool byte_array_get_prop      (JSContext    *context,
                                         JSObject     *obj,
-                                        jsval         id,
+                                        jsid          id,
                                         jsval        *value_p);
 static JSBool byte_array_set_prop      (JSContext    *context,
                                         JSObject     *obj,
-                                        jsval         id,
+                                        jsid          id,
                                         jsval        *value_p);
 static JSBool byte_array_new_resolve   (JSContext    *context,
                                         JSObject     *obj,
-                                        jsval         id,
+                                        jsid          id,
                                         uintN         flags,
                                         JSObject    **objp);
 static JSBool byte_array_constructor   (JSContext    *context,
@@ -98,7 +99,7 @@ gjs_value_from_gsize(JSContext         *context,
         *value_p = INT_TO_JSVAL(v);
         return JS_TRUE;
     } else {
-        return JS_NewDoubleValue(context, v, value_p);
+        return JS_NewNumberValue(context, v, value_p);
     }
 }
 
@@ -107,6 +108,8 @@ gjs_value_to_gsize(JSContext         *context,
                    jsval              value,
                    gsize             *v_p)
 {
+    guint32 val32;
+
     /* Just JS_ValueToECMAUint32() would work. However,
      * we special case ints for two reasons:
      *  - JS_ValueToECMAUint32() always goes via a double which is slow
@@ -122,11 +125,15 @@ gjs_value_to_gsize(JSContext         *context,
         *v_p = i;
         return JS_TRUE;
     } else {
+        JSBool ret;
         /* This is pretty liberal (it converts about anything to
          * a number) but it's what we use elsewhere in gjs too.
          */
-        return JS_ValueToECMAUint32(context, value,
-                                    v_p);
+
+        ret = JS_ValueToECMAUint32(context, value,
+                                   &val32);
+        *v_p = val32;
+        return ret;
     }
 }
 
@@ -142,7 +149,7 @@ gjs_value_to_byte(JSContext         *context,
 
     if (v >= 256) {
         gjs_throw(context,
-                  "Value %u is not a valid byte; must be in range [0,255]",
+                  "Value %" G_GSIZE_FORMAT " is not a valid byte; must be in range [0,255]",
                   v);
         return JS_FALSE;
     }
@@ -162,7 +169,7 @@ byte_array_get_index(JSContext         *context,
 
     if (idx >= priv->array->len) {
         gjs_throw(context,
-                  "Index %u is out of range for ByteArray length %u",
+                  "Index %" G_GSIZE_FORMAT " is out of range for ByteArray length %u",
                   idx,
                   priv->array->len);
         return JS_FALSE;
@@ -180,10 +187,11 @@ byte_array_get_index(JSContext         *context,
 static JSBool
 byte_array_get_prop(JSContext *context,
                     JSObject  *obj,
-                    jsval      id,
+                    jsid       id,
                     jsval     *value_p)
 {
     ByteArrayInstance *priv;
+    jsval id_value;
 
     priv = priv_from_js(context, obj);
 
@@ -192,10 +200,13 @@ byte_array_get_prop(JSContext *context,
     if (priv->array == NULL)
         return JS_TRUE; /* prototype, not an instance. */
 
+    if (!JS_IdToValue(context, id, &id_value))
+        return JS_FALSE;
+
     /* First handle array indexing */
-    if (JSVAL_IS_NUMBER(id)) {
-        unsigned int idx;
-        if (!gjs_value_to_gsize(context, id, &idx))
+    if (JSVAL_IS_NUMBER(id_value)) {
+        gsize idx;
+        if (!gjs_value_to_gsize(context, id_value, &idx))
             return JS_FALSE;
         return byte_array_get_index(context, obj, priv, idx, value_p);
     }
@@ -210,7 +221,7 @@ byte_array_get_prop(JSContext *context,
 static JSBool
 byte_array_length_getter(JSContext *context,
                          JSObject  *obj,
-                         jsval      id,
+                         jsid       id,
                          jsval     *value_p)
 {
     ByteArrayInstance *priv;
@@ -229,11 +240,11 @@ byte_array_length_getter(JSContext *context,
 static JSBool
 byte_array_length_setter(JSContext *context,
                          JSObject  *obj,
-                         jsval      id,
+                         jsid       id,
                          jsval     *value_p)
 {
     ByteArrayInstance *priv;
-    guint32 len = 0;
+    gsize len = 0;
 
     priv = priv_from_js(context, obj);
 
@@ -288,10 +299,11 @@ byte_array_set_index(JSContext         *context,
 static JSBool
 byte_array_set_prop(JSContext *context,
                     JSObject  *obj,
-                    jsval      id,
+                    jsid       id,
                     jsval     *value_p)
 {
     ByteArrayInstance *priv;
+    jsval id_value;
 
     priv = priv_from_js(context, obj);
 
@@ -300,10 +312,13 @@ byte_array_set_prop(JSContext *context,
     if (priv->array == NULL)
         return JS_TRUE; /* prototype, not an instance. */
 
+    if (!JS_IdToValue(context, id, &id_value))
+        return JS_FALSE;
+
     /* First handle array indexing */
-    if (JSVAL_IS_NUMBER(id)) {
+    if (JSVAL_IS_NUMBER(id_value)) {
         gsize idx;
-        if (!gjs_value_to_gsize(context, id, &idx))
+        if (!gjs_value_to_gsize(context, id_value, &idx))
             return JS_FALSE;
 
         return byte_array_set_index(context, obj, priv, idx, value_p);
@@ -338,11 +353,12 @@ byte_array_set_prop(JSContext *context,
 static JSBool
 byte_array_new_resolve(JSContext *context,
                        JSObject  *obj,
-                       jsval      id,
+                       jsid       id,
                        uintN      flags,
                        JSObject **objp)
 {
     ByteArrayInstance *priv;
+    jsval id_val;
 
     priv = priv_from_js(context, *objp);
 
@@ -351,9 +367,12 @@ byte_array_new_resolve(JSContext *context,
     if (priv->array == NULL)
         return JS_TRUE; /* prototype, not an instance. */
 
-    if (JSVAL_IS_NUMBER(id)) {
-        unsigned int idx;
-        if (!gjs_value_to_gsize(context, id, &idx))
+    if (!JS_IdToValue(context, id, &id_val))
+        return JS_FALSE;
+
+    if (JSVAL_IS_NUMBER(id_val)) {
+        gsize idx;
+        if (!gjs_value_to_gsize(context, id_val, &idx))
             return JS_FALSE;
         if (idx >= priv->array->len) {
             *objp = NULL;
@@ -425,7 +444,7 @@ byte_array_constructor(JSContext *context,
     gboolean is_proto;
     JSClass *obj_class;
     JSClass *proto_class;
-    guint32 preallocated_length;
+    gsize preallocated_length;
 
     if (!gjs_check_constructing(context))
         return JS_FALSE;
@@ -485,18 +504,16 @@ byte_array_finalize(JSContext *context,
 /* implement toString() with an optional encoding arg */
 static JSBool
 to_string_func(JSContext *context,
-               JSObject  *obj,
                uintN      argc,
-               jsval     *argv,
-               jsval     *retval)
+               jsval     *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
+    JSObject *object = JS_THIS_OBJECT(context, vp);
     ByteArrayInstance *priv;
     const char *encoding;
     gboolean encoding_is_utf8;
 
-    *retval = JSVAL_VOID;
-
-    priv = priv_from_js(context, obj);
+    priv = priv_from_js(context, object);
 
     if (priv == NULL)
         return JS_FALSE; /* wrong class passed in */
@@ -525,12 +542,18 @@ to_string_func(JSContext *context,
         /* optimization, avoids iconv overhead and runs
          * glib's hardwired utf8-to-utf16
          */
-        return gjs_string_from_utf8(context,
-                                    (char*) priv->array->data,
-                                    priv->array->len,
-                                    retval);
-    } else {
+        jsval retval;
         JSBool ok;
+
+        ok = gjs_string_from_utf8(context,
+                                  (char*) priv->array->data,
+                                  priv->array->len,
+                                  &retval);
+        if (ok)
+            JS_SET_RVAL(context, vp, retval);
+        return ok;
+    } else {
+        JSBool ok = JS_FALSE;
         gsize bytes_written;
         GError *error;
         JSString *s;
@@ -558,12 +581,9 @@ to_string_func(JSContext *context,
         s = JS_NewUCStringCopyN(context,
                                 (jschar*) u16_str,
                                 bytes_written / 2);
-        if (s == NULL) {
-            ok = FALSE;
-            *retval = JSVAL_VOID;
-        } else {
-            ok = TRUE;
-            *retval = STRING_TO_JSVAL(s);
+        if (s != NULL) {
+            ok = JS_TRUE;
+            JS_SET_RVAL(context, vp, STRING_TO_JSVAL(s));
         }
 
         g_free(u16_str);
@@ -572,8 +592,7 @@ to_string_func(JSContext *context,
 }
 
 static JSObject*
-byte_array_new(JSContext *context,
-               JSObject  *module)
+byte_array_new(JSContext *context)
 {
     JSObject *array;
     ByteArrayInstance *priv;
@@ -592,34 +611,31 @@ byte_array_new(JSContext *context,
 /* fromString() function implementation */
 static JSBool
 from_string_func(JSContext *context,
-                 JSObject  *module,
                  uintN      argc,
-                 jsval     *argv,
-                 jsval     *retval)
+                 jsval     *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
     ByteArrayInstance *priv;
     const char *encoding;
     gboolean encoding_is_utf8;
     JSObject *obj;
+    JSBool retval = JS_FALSE;
 
-    *retval = JSVAL_VOID;
-
-    obj = byte_array_new(context, module);
+    obj = byte_array_new(context);
     if (obj == NULL)
         return JS_FALSE;
-    *retval = OBJECT_TO_JSVAL(obj); /* side effect: roots obj */
+
+    JS_AddObjectRoot(context, &obj);
 
     priv = priv_from_js(context, obj);
-
-    if (priv == NULL)
-        return JS_FALSE; /* wrong class passed in */
+    g_assert (priv != NULL);
 
     g_assert(argc > 0); /* because we specified min args 1 */
 
     if (!JSVAL_IS_STRING(argv[0])) {
         gjs_throw(context,
                   "byteArray.fromString() called with non-string as first arg");
-        return JS_FALSE;
+        goto out;
     }
 
     encoding_is_utf8 = TRUE;
@@ -627,7 +643,7 @@ from_string_func(JSContext *context,
         JSVAL_IS_STRING(argv[1])) {
         encoding = gjs_string_get_ascii_checked(context, argv[1]);
         if (encoding == NULL)
-            return JS_FALSE;
+            goto out;
 
         /* maybe we should be smarter about utf8 synonyms here.
          * doesn't matter much though. encoding_is_utf8 is
@@ -654,7 +670,7 @@ from_string_func(JSContext *context,
         if (!gjs_string_to_utf8(context,
                                 argv[0],
                                 &utf8))
-            return JS_FALSE;
+            goto out;
 
         g_byte_array_set_size(priv->array, 0);
         g_byte_array_append(priv->array, (guint8*) utf8, strlen(utf8));
@@ -680,7 +696,7 @@ from_string_func(JSContext *context,
         if (encoded == NULL) {
             /* frees the GError */
             gjs_throw_g_error(context, error);
-            return JS_FALSE;
+            goto out;
         }
 
         g_byte_array_set_size(priv->array, 0);
@@ -688,48 +704,49 @@ from_string_func(JSContext *context,
 
         g_free(encoded);
     }
+    
+    JS_SET_RVAL(context, vp, OBJECT_TO_JSVAL(obj));
 
-    return JS_TRUE;
+    retval = JS_TRUE;
+ out:
+    JS_RemoveObjectRoot(context, &obj);
+    return retval;
 }
 
 /* fromArray() function implementation */
 static JSBool
 from_array_func(JSContext *context,
-                JSObject  *module,
                 uintN      argc,
-                jsval     *argv,
-                jsval     *retval)
+                jsval     *vp)
 {
+    jsval *argv = JS_ARGV(context, vp);
     ByteArrayInstance *priv;
     jsuint len;
     jsuint i;
     JSObject *obj;
+    JSBool ret = JS_FALSE;
 
-    *retval = JSVAL_VOID;
-
-    obj = byte_array_new(context, module);
+    obj = byte_array_new(context);
     if (obj == NULL)
         return JS_FALSE;
 
-    *retval = OBJECT_TO_JSVAL(obj); /* side effect: roots obj */
+    JS_AddObjectRoot(context, &obj);
 
     priv = priv_from_js(context, obj);
-
-    if (priv == NULL)
-        return JS_FALSE; /* wrong class passed in */
+    g_assert (priv != NULL);
 
     g_assert(argc > 0); /* because we specified min args 1 */
 
     if (!JS_IsArrayObject(context, JSVAL_TO_OBJECT(argv[0]))) {
         gjs_throw(context,
                   "byteArray.fromArray() called with non-array as first arg");
-        return JS_FALSE;
+        goto out;
     }
 
     if (!JS_GetArrayLength(context, JSVAL_TO_OBJECT(argv[0]), &len)) {
         gjs_throw(context,
                   "byteArray.fromArray() can't get length of first array arg");
-        return JS_FALSE;
+        goto out;
     }
 
     g_byte_array_set_size(priv->array, len);
@@ -743,19 +760,23 @@ from_array_func(JSContext *context,
             /* this means there was an exception, while elem == JSVAL_VOID
              * means no element found
              */
-            return JS_FALSE;
+            goto out;
         }
 
         if (elem == JSVAL_VOID)
             continue;
 
         if (!gjs_value_to_byte(context, elem, &b))
-            return JS_FALSE;
+            goto out;
 
         g_array_index(priv->array, guint8, i) = b;
     }
 
-    return JS_TRUE;
+    ret = JS_TRUE;
+    JS_SET_RVAL(context, vp, OBJECT_TO_JSVAL(obj));
+ out:
+    JS_RemoveObjectRoot(context, &obj);
+    return ret;
 }
 
 JSObject *
@@ -823,13 +844,13 @@ static JSPropertySpec gjs_byte_array_proto_props[] = {
 };
 
 static JSFunctionSpec gjs_byte_array_proto_funcs[] = {
-    { "toString", to_string_func, 0, 0 },
+    { "toString", (JSNative) to_string_func, 0, JSFUN_FAST_NATIVE },
     { NULL }
 };
 
 static JSFunctionSpec gjs_byte_array_module_funcs[] = {
-    { "fromString", from_string_func, 1, 0 },
-    { "fromArray", from_array_func, 1, 0 },
+    { "fromString", (JSNative)from_string_func, 1, JSFUN_FAST_NATIVE },
+    { "fromArray", (JSNative)from_array_func, 1, JSFUN_FAST_NATIVE },
     { NULL }
 };
 
@@ -837,9 +858,8 @@ JSBool
 gjs_define_byte_array_stuff(JSContext      *context,
                             JSObject       *in_object)
 {
-    JSContext *load_context = gjs_runtime_get_load_context(JS_GetRuntime(context));
-    JSObject *global = JS_GetGlobalObject(context);
-    gjs_byte_array_prototype = JS_InitClass(load_context, global,
+    JSObject *global = gjs_get_import_global(context);
+    gjs_byte_array_prototype = JS_InitClass(context, global,
                              NULL,
                              &gjs_byte_array_class,
                              byte_array_constructor,
@@ -850,17 +870,13 @@ gjs_define_byte_array_stuff(JSContext      *context,
                              NULL);
     jsval rval;
 
-    if (gjs_byte_array_prototype == NULL) {
-        gjs_move_exception(load_context, context);
+    if (gjs_byte_array_prototype == NULL)
         return JS_FALSE;
-    }
 
     if (!gjs_object_require_property(
-            load_context, global, NULL,
-            "ByteArray", &rval)) {
-        gjs_move_exception(load_context, context);
+            context, global, NULL,
+            "ByteArray", &rval))
         return JS_FALSE;
-    }
 
     if (!JS_DefineProperty(context, in_object, "ByteArray",
                            rval, NULL, NULL, GJS_MODULE_PROP_FLAGS))
