@@ -24,7 +24,6 @@
 #include <config.h>
 
 #include "context.h"
-#include "context-jsapi.h"
 #include "importer.h"
 #include "jsapi-util.h"
 #include "profiler.h"
@@ -65,7 +64,6 @@ struct _GjsContext {
     char **search_path;
 
     unsigned int we_own_runtime : 1;
-    unsigned int is_load_context : 1;
 };
 
 struct _GjsContextClass {
@@ -85,8 +83,7 @@ static int signals[LAST_SIGNAL];
 enum {
     PROP_0,
     PROP_SEARCH_PATH,
-    PROP_RUNTIME,
-    PROP_IS_LOAD_CONTEXT
+    PROP_RUNTIME
 };
 
 
@@ -200,7 +197,7 @@ gjs_print_parse_args(JSContext *context,
     JS_BeginRequest(context);
 
     str = g_string_new("");
-    JS_EnterLocalRootScope(context);
+    (void)JS_EnterLocalRootScope(context);
     for (n = 0; n < argc; ++n) {
         JSExceptionState *exc_state;
         JSString *jstr;
@@ -325,10 +322,6 @@ gjs_context_class_init(GjsContextClass *klass)
                                  FALSE,
                                  G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
-    g_object_class_install_property(object_class,
-                                    PROP_IS_LOAD_CONTEXT,
-                                    pspec);
-
     gjs_register_native_module("byteArray", gjs_define_byte_array_stuff, 0);
 }
 
@@ -351,37 +344,21 @@ gjs_context_dispose(GObject *object)
     if (js_context->context != NULL) {
 
         gjs_debug(GJS_DEBUG_CONTEXT,
-                  "Destroying JS context%s",
-                  js_context->is_load_context ? " (load context)" : "");
+                  "Destroying JS context");
 
+        if (js_context->we_own_runtime)
+            gjs_runtime_set_default_context(js_context->runtime, NULL);
         JS_DestroyContext(js_context->context);
         js_context->context = NULL;
     }
 
     if (js_context->runtime != NULL) {
         if (js_context->we_own_runtime) {
-            /* Avoid keeping JSContext with a dangling pointer to the
-             * runtime.
+            /* Cleans up data as well as destroying the runtime. Foreign
+             * contexts aren't supported at the moment; if we supported
+             * them, then the data simply would't get cleaned up.
              */
-            gjs_runtime_clear_call_context(js_context->runtime);
-            gjs_runtime_clear_load_context(js_context->runtime);
-
-            gjs_debug(GJS_DEBUG_CONTEXT,
-                      "Destroying JS runtime");
-
-            JS_DestroyRuntime(js_context->runtime);
-
-            /* finalize the dataset from jsapi-util.c ...  for
-             * "foreign" runtimes this just never happens for
-             * now... we do this after the runtime itself is destroyed
-             * because we might have finalizers run by
-             * JS_DestroyRuntime() that rely on data we've set on the
-             * runtime, such as the dynamic class structs.
-             */
-            gjs_debug(GJS_DEBUG_CONTEXT,
-                      "Destroying any remaining dataset items on runtime");
-
-            g_dataset_destroy(js_context->runtime);
+            gjs_runtime_destroy(js_context->runtime);
         }
         js_context->runtime = NULL;
     }
@@ -545,6 +522,7 @@ gjs_context_constructor (GType                  type,
             gjs_fatal("Failed to create javascript runtime");
         JS_SetGCParameter(js_context->runtime, JSGC_MAX_BYTES, 0xffffffff);
         js_context->we_own_runtime = TRUE;
+        gjs_runtime_init(js_context->runtime);
     }
 
     js_context->context = JS_NewContext(js_context->runtime, 8192 /* stack chunk size */);
@@ -628,31 +606,37 @@ gjs_context_constructor (GType                  type,
                            4, GJS_MODULE_PROP_FLAGS))
         gjs_fatal("Failed to define printerr function");
 
-    /* If we created the root importer in the load context,
-     * there would be infinite recursion since the load context
-     * is a GjsContext
+    /* We need to know what the default context is, since it's the context whose
+     * global object is used to load imported JS modules. We currently say that
+     * it's the context of the runtime's owner, but if we needed to support
+     * externally created runtimes, we could define it in some other fashion.
      */
-    if (!js_context->is_load_context) {
-        /* We create the global-to-runtime root importer with the
-         * passed-in search path. If someone else already created
-         * the root importer, this is a no-op.
-         */
-        if (!gjs_create_root_importer(js_context->runtime,
-                                      js_context->search_path ?
-                                      (const char**) js_context->search_path :
-                                      NULL,
-                                      TRUE))
-            gjs_fatal("Failed to create root importer");
-
-        /* Now copy the global root importer (which we just created,
-         * if it didn't exist) to our global object
-         */
-        if (!gjs_define_root_importer(js_context->context,
-                                      js_context->global,
-                                      "imports"))
-            gjs_fatal("Failed to point 'imports' property at root importer");
+    if (js_context->we_own_runtime) {
+        gjs_runtime_set_default_context(js_context->runtime, js_context->context);
+    } else {
+        if (gjs_runtime_get_default_context(js_context->runtime) == NULL)
+            gjs_fatal("GjsContext created for a runtime not owned by GJS");
     }
 
+    /* We create the global-to-runtime root importer with the
+     * passed-in search path. If someone else already created
+     * the root importer, this is a no-op.
+     */
+    if (!gjs_create_root_importer(js_context->context,
+                                  js_context->search_path ?
+                                  (const char**) js_context->search_path :
+                                  NULL,
+                                  TRUE))
+        gjs_fatal("Failed to create root importer");
+
+    /* Now copy the global root importer (which we just created,
+     * if it didn't exist) to our global object
+     */
+    if (!gjs_define_root_importer(js_context->context,
+                                  js_context->global,
+                                  "imports"))
+        gjs_fatal("Failed to point 'imports' property at root importer");
+
     if (js_context->we_own_runtime) {
         js_context->profiler = gjs_profiler_new(js_context->runtime);
     }
@@ -677,10 +661,6 @@ gjs_context_get_property (GObject     *object,
     js_context = GJS_CONTEXT (object);
 
     switch (prop_id) {
-    case PROP_IS_LOAD_CONTEXT:
-        g_value_set_boolean(value, js_context->is_load_context);
-        break;
-
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -704,9 +684,6 @@ gjs_context_set_property (GObject      *object,
     case PROP_RUNTIME:
         js_context->runtime = g_value_get_pointer(value);
         break;
-    case PROP_IS_LOAD_CONTEXT:
-        js_context->is_load_context = g_value_get_boolean(value);
-        break;
 
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -763,12 +740,6 @@ gjs_context_get_native_context (GjsContext *js_context)
     return js_context->context;
 }
 
-gboolean
-gjs_context_is_load_context(GjsContext *js_context)
-{
-    return js_context->is_load_context;
-}
-
 gboolean
 gjs_context_eval(GjsContext *js_context,
                  const char   *script,
@@ -815,6 +786,7 @@ gjs_context_eval(GjsContext *js_context,
     /* JS_EvaluateScript requires a request even though it sort of seems like
      * it means we're always in a request?
      */
+    gjs_runtime_push_context(js_context->runtime, js_context->context);
     JS_BeginRequest(js_context->context);
 
     retval = JSVAL_VOID;
@@ -879,9 +851,10 @@ gjs_context_eval(GjsContext *js_context,
         }
     }
 
-    g_object_unref(G_OBJECT(js_context));
-
     JS_EndRequest(js_context->context);
+    gjs_runtime_pop_context(js_context->runtime);
+
+    g_object_unref(G_OBJECT(js_context));
 
     return success;
 }
@@ -949,6 +922,9 @@ gjstest_test_func_gjs_context_construct_destroy(void)
 {
     GjsContext *context;
 
+    /* Construct twice just to possibly a case where global state from
+     * the first leaks.
+     */
     context = gjs_context_new ();
     g_object_unref (context);
 
