@@ -24,8 +24,8 @@
 #ifndef __GJS_JSAPI_UTIL_H__
 #define __GJS_JSAPI_UTIL_H__
 
-#ifndef __GJS_GJS_H__
-#warning Include <gjs/gjs.h> instead of <gjs/jsapi-util.h>
+#if !defined (__GJS_GJS_MODULE_H__) && !defined (GJS_COMPILATION)
+#error "Only <gjs/gjs-module.h> can be included directly."
 #endif
 
 #include <jsapi.h>
@@ -164,11 +164,10 @@ static struct JSClass cname##_class = { \
 jsval cname##_create_proto(JSContext *context, JSObject *module, const char *proto_name, JSObject *parent) \
 { \
     jsval rval; \
-    JSContext *load_context = gjs_runtime_get_load_context(JS_GetRuntime(context)); \
-    JSObject *global = JS_GetGlobalObject(context); \
-    if (!gjs_object_has_property(load_context, global, \
+    JSObject *global = gjs_get_import_global(context); \
+    if (!gjs_object_has_property(context, global, \
                                  cname##_class.name)) { \
-        JSObject *prototype = JS_InitClass(load_context, global, \
+        JSObject *prototype = JS_InitClass(context, global, \
                                  parent, \
                                  &cname##_class, \
                                  ctor, \
@@ -178,13 +177,11 @@ jsval cname##_create_proto(JSContext *context, JSObject *module, const char *pro
                                  NULL, \
                                  NULL); \
         if (prototype == NULL) { \
-            gjs_move_exception(load_context, context); \
             return JSVAL_NULL; \
         } \
         if (!gjs_object_require_property( \
-                load_context, global, NULL, \
+                context, global, NULL, \
                 cname##_class.name, &rval)) { \
-            gjs_move_exception(load_context, context); \
             return JSVAL_NULL; \
         } \
     } \
@@ -194,18 +191,19 @@ jsval cname##_create_proto(JSContext *context, JSObject *module, const char *pro
     return rval; \
 }
 
+gboolean    gjs_init_context_standard        (JSContext       *context);
 
-void*       gjs_runtime_get_data             (JSRuntime       *runtime,
-                                                 const char      *name);
-void        gjs_runtime_set_data             (JSRuntime       *runtime,
-                                                 const char      *name,
-                                                 void            *data,
-                                                 GDestroyNotify   dnotify);
-JSContext*  gjs_runtime_get_load_context     (JSRuntime       *runtime);
-JSContext*  gjs_runtime_peek_load_context    (JSRuntime       *runtime);
-void        gjs_runtime_clear_load_context   (JSRuntime       *runtime);
-JSContext*  gjs_runtime_get_call_context     (JSRuntime       *runtime);
-void        gjs_runtime_clear_call_context   (JSRuntime       *runtime);
+void        gjs_runtime_init                 (JSRuntime       *runtime);
+void        gjs_runtime_destroy              (JSRuntime       *runtime);
+
+JSContext*  gjs_runtime_get_current_context  (JSRuntime       *runtime);
+void        gjs_runtime_set_default_context  (JSRuntime       *runtime,
+                                              JSContext       *context);
+JSContext*  gjs_runtime_get_default_context  (JSRuntime       *runtime);
+void        gjs_runtime_push_context         (JSRuntime       *runtime,
+                                              JSContext       *context);
+void        gjs_runtime_pop_context          (JSRuntime       *runtime);
+JSObject*   gjs_get_import_global            (JSContext       *context);
 gboolean    gjs_object_has_property          (JSContext       *context,
                                               JSObject        *obj,
                                               const char      *property_name);
@@ -331,7 +329,8 @@ JSBool      gjs_string_get_uint16_data       (JSContext       *context,
                                               jsval            value,
                                               guint16        **data_p,
                                               gsize           *len_p);
-JSBool      gjs_get_string_id                (jsval            id_val,
+JSBool      gjs_get_string_id                (JSContext       *context,
+                                              jsid             id,
                                               const char     **name_p);
 const char* gjs_get_type_name                (jsval            value);
 
